@@ -22,45 +22,52 @@ sub as_string {
     while ( my ( $key, $value ) = each %{ $headers || {} } ) {
         $frame .= $key . ':' . $value . "\n";
     }
-
-    $body ||= '';
-    return "$frame\n$body\000";
+    $frame .= "\n";
+    $frame .= $body || '';
+    $frame .= "\000";
 }
 
 sub parse {
-    my ( $class, $string ) = @_;
-    my ( $command, $headers, $body );
-
-    my @failure = ( $string, undef );
-
-    ( $command, $string ) = split( "\n", $string, 2 );
-    return @failure unless $command;
+    my ( $package, $socket ) = @_;
+    local $/ = "\n";
 
+    # read the command
+    my $command;
     while (1) {
-        my ( $header, $key, $val );
-        ( $header, $string ) = split( "\n", $string, 2 );
-        last unless $header;
-        ( $key, $val ) = split( /: */, $header, 2 );
-        $headers->{$key} = $val;
+        $command = $socket->getline || die "Error reading command: $!";
+        chop $command;
+        last if $command;
     }
 
-    if ( my $length = $headers->{'content-length'} ) {
-        return @failure unless length($string) >= $length;
+    # read headers
+    my $headers;
+    while (1) {
+        my $line = $socket->getline || die "Error reading header: $!";
+        chop $line;
+        last if $line eq "";
+        my ( $key, $value ) = split /: ?/, $line, 2;
+        $headers->{$key} = $value;
+    }
+
+    # read the body
+    my $body;
+    if ( $headers->{"content-length"} ) {
+        $socket->read( $body, $headers->{"content-length"} )
+            || die "Error reading body: $!";
+        $socket->getc;    # eat the trailing null
         $headers->{bytes_message} = 1;
-        $body = substr( $string, 0, $length );
-        $string = substr( $string, $length + 1 );    # +1 to eat trailing null
     } else {
-        return @failure unless $string =~ "\000";
-        ( $body, $string ) = split( "\000", $string, 2 );
+        while (1) {
+            my $byte = $socket->getc;
+            die "Error reading body: $!" unless defined $byte;
+            last if $byte eq "\000";
+            $body .= $byte;
+        }
     }
 
     my $frame = Net::Stomp::Frame->new(
-        {   command => $command,
-            headers => $headers,
-            body    => $body,
-        }
-    );
-    return ( $string, $frame );
+        { command => $command, headers => $headers, body => $body } );
+    return $frame;
 }
 
 1;
@@ -110,22 +117,13 @@ Create a new L<Net::Stomp::Frame> object:
 
 =head2 parse
 
-Create a new L<Net::Stomp::Frame> given a string containing the serialised 
-frame:
+Create a new L<Net::Somp::Frame> given a string containing the serialised frame:
 
   my $frame  = Net::Stomp::Frame->parse($string);
 
-If called in scalar context as above, the frame is returned (or undef if the
-frame couldn't be parsed).  
-
-In list context, the remainder of the string left over from parsing and the 
-frame are returned (or in failure mode, the original string and undef).
-
-  my ($remaining, $frame) = Net::Stomp::Frame->parse($string);
-
 =head2 as_string
 
-Create a string representing the serialized frame:
+Create a string containing the serialised frame representing the frame:
 
   my $string = $frame->as_string;
 
